unit XFontComboBox;

{
  TXFontComboBox 1.0 (1999-10-04)
  ===============================
  This component enables you to select font.

  Freeware.

  Copyright  1999 Roman Stedronsky, Roman.Stedronsky@seznam.cz.

  All rights reserved. You may use this software in an application
  without fee or royalty, provided this copyright notice remains intact.

  public methods:
    ReloadFonts		clears current font list and builds new one

  for getting font use standard Items property (but don't change it!)
}

{$S-,W-,R-}
{$C PRELOAD}

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, StdCtrls;

type
  TXFontComboBox = class(TCustomComboBox)
  private
    FComboBoxEdit: boolean;
  protected
    procedure CreateWnd; override;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ReloadFonts;
  published
    // publish propeties from TXustomComboBox
    property Anchors;
    property BiDiMode;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure TXFontComboBox.CreateWnd;
begin
  inherited CreateWnd;
  Perform(CB_SETITEMHEIGHT, 0, 24);
  Perform(CB_SETDROPPEDWIDTH, 244, 0);
  ReloadFonts;
end;

procedure TXFontComboBox.CNDrawItem(var Message: TWMDrawItem);
begin
  FComboBoxEdit := Message.DrawItemStruct.itemState and ODS_COMBOBOXEDIT <> 0;
  inherited;
end;

procedure TXFontComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  TControlCanvas(Canvas).UpdateTextFlags;
  if odSelected in State then
  begin
    Canvas.Brush.Color := clHighlight;
  end;
  Canvas.FillRect(Rect);

  if odFocused in State then
  begin
    Canvas.Brush.Color := clHighlightText;
    Canvas.DrawFocusRect(Rect);
  end;

  InflateRect(Rect, -1, -1);
  if odSelected in State then
  begin
    Canvas.Pen.Color := clHighlightText;
    Canvas.Brush.Color := clHighlight;
  end
  else
  begin
    Canvas.Pen.Color := Font.Color;
    Canvas.Brush.Color := Color;
  end;
  if FComboBoxEdit then
  begin
    Canvas.Font.Size := Font.Size;
    Canvas.Font.Name := Font.Name;
  end
  else
  begin
    Canvas.Font.Height := Perform(CB_GETITEMHEIGHT, 0, 0) - 2;
    Canvas.Font.Name := Items[Index];
  end;
  DrawText(Canvas.Handle, PChar(Items[Index]), Length(Items[Index]), Rect, DT_VCENTER or DT_LEFT or DT_SINGLELINE);
end;

constructor TXFontComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
end;

procedure TXFontComboBox.ReloadFonts;
var
  Old: string;
  Index: integer;
begin
  // and what about WM_FONTCHANGE reaction ?
  Items.BeginUpdate;
  if ItemIndex <> -1 then
    Old := Items[ItemIndex];
  Items.Clear;
  Items.AddStrings(Screen.Fonts);
  Items.EndUpdate;
  Index := Items.IndexOf(Old);
  if Index <> -1 then
    ItemIndex := Index
  else
    ItemIndex := 0;
end;

{ registration }

procedure Register;
begin
  RegisterComponents('Extra', [TXFontComboBox]);
end;

end.

